@@ -16,11 +16,14 @@
 
    Note, this is not iSCSI.
 """
+
+from __future__ import annotations
+
 import glob
 import os
 import re
 import time
-from typing import Dict, List, Optional  # noqa: H301
+from typing import Optional
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
@@ -82,7 +85,7 @@ class LinuxSCSI(executor.Executor):
             with exc.context(force, 'Removing %s failed', device):
                 self.echo_scsi_command(path, "1")
 
-    def wait_for_volumes_removal(self, volumes_names: List[str]) -> None:
+    def wait_for_volumes_removal(self, volumes_names: list[str]) -> None:
         """Wait for device paths to be removed from the system."""
         str_names = ', '.join(volumes_names)
         LOG.debug('Checking to see if SCSI volumes %s have been removed.',
@@ -105,7 +108,7 @@ class LinuxSCSI(executor.Executor):
                     LOG.debug('%s still exist.', ', '.join(exist))
         raise exception.VolumePathNotRemoved(volume_path=exist)
 
-    def get_device_info(self, device: str) -> Dict[str, Optional[str]]:
+    def get_device_info(self, device: str) -> dict[str, Optional[str]]:
         dev_info = {'device': device, 'host': None,
                     'channel': None, 'id': None, 'lun': None}
         # The input argument 'device' can be of 2 types:
@@ -132,7 +135,7 @@ class LinuxSCSI(executor.Executor):
         LOG.debug('dev_info=%s', str(dev_info))
         return dev_info
 
-    def get_sysfs_wwn(self, device_names, mpath=None) -> str:
+    def get_sysfs_wwn(self, device_names: list[str], mpath=None) -> str:
         """Return the wwid from sysfs in any of devices in udev format."""
         # If we have a multipath DM we know that it has found the WWN
         if mpath:
@@ -154,7 +157,7 @@ class LinuxSCSI(executor.Executor):
             return wwid
 
         # If we have multiple designators use symlinks to find out the wwn
-        device_names = set(device_names)
+        device_names_set = set(device_names)
         for wwn_path in wwn_paths:
             try:
                 if os.path.islink(wwn_path) and os.stat(wwn_path):
@@ -170,11 +173,11 @@ class LinuxSCSI(executor.Executor):
                             dm_devs = os.listdir(slaves_path)
                             # This is the right wwn_path if the devices we have
                             # attached belong to the dm we followed
-                            if device_names.intersection(dm_devs):
+                            if device_names_set.intersection(dm_devs):
                                 break
 
                         # This is the right wwn_path if  devices we have
-                        elif name in device_names:
+                        elif name in device_names_set:
                             break
             except OSError:
                 continue
@@ -197,7 +200,7 @@ class LinuxSCSI(executor.Executor):
             return udev_wwid
         return ''
 
-    def get_scsi_wwn(self, path):
+    def get_scsi_wwn(self, path: str) -> str:
         """Read the WWN from page 0x83 value for a SCSI device."""
 
         (out, _err) = self._execute('/lib/udev/scsi_id', '--page', '0x83',
@@ -549,17 +552,6 @@ class LinuxSCSI(executor.Executor):
             return info
         return None
 
-    def get_device_size(self, device):
-        """Get the size in bytes of a volume."""
-        (out, _err) = self._execute('blockdev', '--getsize64',
-                                    device, run_as_root=True,
-                                    root_helper=self._root_helper)
-        var = str(out.strip())
-        if var.isnumeric():
-            return int(var)
-        else:
-            return None
-
     def multipath_reconfigure(self):
         """Issue a multipathd reconfigure.
 
@@ -612,7 +604,9 @@ class LinuxSCSI(executor.Executor):
                         ctx.reraise = False
                         time.sleep(1)
 
-    def extend_volume(self, volume_paths, use_multipath=False):
+    def extend_volume(self,
+                      volume_paths: list,
+                      use_multipath: bool = False) -> Optional[int]:
         """Signal the SCSI subsystem to test for volume resize.
 
         This function tries to signal the local system's kernel
@@ -632,13 +626,13 @@ class LinuxSCSI(executor.Executor):
             scsi_path = ("/sys/bus/scsi/drivers/sd/%(device_id)s" %
                          {'device_id': device_id})
 
-            size = self.get_device_size(volume_path)
+            size = utils.get_device_size(self, volume_path)
             LOG.debug("Starting size: %s", size)
 
             # now issue the device rescan
             rescan_path = "%(scsi_path)s/rescan" % {'scsi_path': scsi_path}
             self.echo_scsi_command(rescan_path, "1")
-            new_size = self.get_device_size(volume_path)
+            new_size = utils.get_device_size(self, volume_path)
             LOG.debug("volume size after scsi device rescan %s", new_size)
 
         scsi_wwn = self.get_scsi_wwn(volume_paths[0])
@@ -648,13 +642,13 @@ class LinuxSCSI(executor.Executor):
                 # Force a reconfigure so that resize works
                 self.multipath_reconfigure()
 
-                size = self.get_device_size(mpath_device)
+                size = utils.get_device_size(self, mpath_device)
                 LOG.info("mpath(%(device)s) current size %(size)s",
                          {'device': mpath_device, 'size': size})
 
                 self.multipath_resize_map(scsi_wwn)
 
-                new_size = self.get_device_size(mpath_device)
+                new_size = utils.get_device_size(self, mpath_device)
                 LOG.info("mpath(%(device)s) new size %(size)s",
                          {'device': mpath_device, 'size': new_size})
 
@@ -765,3 +759,21 @@ class LinuxSCSI(executor.Executor):
                                        check_exit_code=False,
                                        root_helper=self._root_helper)
         return stdout.strip() == 'ok'
+
+    @utils.retry((putils.ProcessExecutionError, exception.BrickException),
+                 retries=3)
+    def multipath_del_map(self, mpath):
+        """Stop monitoring a multipath given its device name (eg: dm-7).
+
+        Method ensures that the multipath device mapper actually dissapears
+        from sysfs.
+        """
+        map_name = self.get_dm_name(mpath)
+        if map_name:
+            self._execute('multipathd', 'del', 'map', map_name,
+                          run_as_root=True, timeout=5,
+                          root_helper=self._root_helper)
+
+        if map_name and self.get_dm_name(mpath):
+            raise exception.BrickException("Multipath doesn't go away")
+        LOG.debug('Multipath %s no longer present', mpath)
